"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const exports_1 = require("parcel-plugin-typescript/exports");
const template_1 = require("../../frontend/loaders/template");
class TypeScriptWorker extends exports_1.Worker {
    constructor(bundler) {
        // We append the socket path to process.env beforce spawning the worker
        exports_1.setSocketPath('angular');
        super(require.resolve('./launcher'));
        this.bundler = bundler;
        this.resources = new Map();
    }
    compile(data) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { resources, sources } = yield this.request('compile', data);
            const result = {
                sources,
                resources: {
                    bundled: resources,
                    external: resources
                        .map(resource => template_1.getFileResources(resource))
                        .reduce((a, b) => a.concat(b), [])
                }
            };
            for (const dep of resources.concat(result.resources.external)) {
                const deps = this.resources.get(dep);
                if (!deps) {
                    this.resources.set(dep, [data.file]);
                }
                else {
                    deps.push(data.file);
                }
            }
            return result;
        });
    }
    typeCheck(data) {
        return this.request('typeCheck', data);
    }
    readVirtualFile(file) {
        return this.request('readVirtualFile', file);
    }
    processResource(file) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { package: pkg, options, parser } = this.bundler;
            const source = yield exports_1.readFile(file);
            return template_1.processResource(file, pkg, options, parser, source);
        });
    }
    invalidate(files) {
        return this.request('invalidate', files);
    }
}
tslib_1.__decorate([
    exports_1.HandlerMethod
], TypeScriptWorker.prototype, "compile", null);
tslib_1.__decorate([
    exports_1.HandlerMethod
], TypeScriptWorker.prototype, "typeCheck", null);
tslib_1.__decorate([
    exports_1.HandlerMethod
], TypeScriptWorker.prototype, "readVirtualFile", null);
tslib_1.__decorate([
    exports_1.HandlerMethod
], TypeScriptWorker.prototype, "processResource", null);
tslib_1.__decorate([
    exports_1.HandlerMethod
], TypeScriptWorker.prototype, "invalidate", null);
exports.TypeScriptWorker = TypeScriptWorker;
class AngularServer extends exports_1.Server {
    constructor(bundler) {
        const worker = new TypeScriptWorker(bundler);
        super('angular', worker);
        this.worker = worker;
        process.nextTick(() => {
            if (bundler.watcher) {
                bundler.watcher.on('change', (file) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                    worker.invalidate([file]);
                    // TODO: trigger before compilation, not after
                    yield worker.request('wait', undefined);
                    const deps = worker.resources.get(file);
                    const files = [];
                    if (deps) {
                        files.push(...deps);
                    }
                    // TODO: batch this
                    worker.invalidate(files);
                }));
            }
        });
    }
    close() {
        this.worker.kill();
        super.close();
    }
}
exports.AngularServer = AngularServer;
//# sourceMappingURL=index.js.map